from mesh import *
from vector import *


class Wire:
    @classmethod
    def skew_quad(cls):
        return Wire(Vector([1,0,1]), Vector([-1,0,1]), Vector([0,1,1]), Vector([0,1,-1]))
           
    def __init__(self, verts):
        """takes a list of vertices as input"""
        self.n = len(verts)
        self.vertices = verts

    def seed_one_point(self, pt):
        for i in range(self.n):
            n = self.n
            #verts 0-> n-1 are the wire. vert n is the seed point
            verts = self.vertices + [Vertex(pt)]
            #edges 0->(n-1) are the wire
            edges = [Edge(verts[i], verts[(i+1)%(n)]) for i in range(n)]
            #edges n->(2n-1) connect to the seed point
            edges += [Edge(verts[i], verts[n]) for i in range(n)]
            faces = [Face(edges[i], edges[n +(1+i)%n], edges[i+n]) for i in range(n)]

            mesh = Mesh(verts, edges, faces)
            bub  = Bubble([self], mesh)
            mesh.set_bubble(bub)
            return bub

    def add_vert(self, vert):
        self.vertices.append(vert)
        self.n += 1

class Bubble:
    @classmethod
    def disc(cls):
        """returns a bubble on a circular wire"""
        angles = [2*math.pi * i/20 for i in range(20)]
        wire_points = [Vertex(Vector([math.cos(t), math.sin(t), 0])) for t in angles]
        wire = Wire(wire_points)
        return wire.seed_one_point(Vector([0,0,1]))

    """@classmethod
    def cylinder(cls, z_steps, segments):
        angles = [2*math.pi * i/segments for i in range(segments)]
        bottom_wire_points = [Vertex(Vector([math.cos(t), math.sin(t), -0.5])) for t in angles]
        top_wire_points = [Vertex(Vector([math.cos(t), math.sin(t), 0.5])) for t in angles]
        wire = Wire(top_wire_points + bottom_wire_points)
        #verts wrap around and then go up
        verts = [Vertex(Vector([math.cos(t), math.sin(t), ((z_steps)/w) - 0.5])) for w in range(z_steps+1) for t in angles]
        #vertical horizontal and diagonal edges
        v_edges = [Edge(verts[i], verts[i+segments]) for i in range(segments*z_steps)]
        h_edges = [Edge(verts[i], verts[i+1]) for i in range(segments*(z_steps+1))]
        d_edges = [Edge(verts[i], verts[i+segments+1]) for i in range(segments*z_steps)]
        
        u_faces = [Face(v_edges[i], 
        return wire.seed_one_point(Vector([0,0,1]))
        """

    @classmethod
    def skirt(cls, segments, lumps):
        """returns a bubble on an oscillating circular wire"""
        angles = [2*math.pi * i/segments for i in range(segments)]
        wire_points = [Vertex(Vector([math.cos(t), math.sin(t), math.sin(lumps*t) + math.sin(3*t)])) for t in angles]
        wire = Wire(wire_points)
        return wire.seed_one_point(Vector([0,0,1]))


    @classmethod
    def load_obj(cls, obj_string):  
        verts=[]
        obj_lines = obj_string.split("\n")
        edge_pairs = []
        edges = []
        faces = []
        v_lines=[]
        v_pointers =[]  #in order of lines, points to the line on which it first occurs
        vs_by_line ={}  #keys are line where it first occured, vals are verts 

        for line in obj_lines:
            if not line: 
                continue
            if line[0]=='#':
                continue

            elif line[0:2]=='v ':
                if line in v_lines:
                    v_pointers.append(v_lines.index(line))
                else:
                    coords = [float(x) for x in line.split(' ')[1:]]
                    vert = Vertex(Vector(coords))
                    verts.append(vert)
                    vs_by_line[len(v_pointers)] = vert
                    v_pointers.append(len(v_pointers))
                v_lines.append(line)

            elif line[0:2]=='f ':
                f_file_indices = [int(a.split('/')[0])-1 for a in line.split(' ')[1:]]
                
                f_verts = [vs_by_line[v_pointers[i]] for i in f_file_indices]
                e=[]

                for i in range(3):
                    if (f_verts[i], f_verts[(i+1)%3]) in edge_pairs:
                        e.append(edges[edge_pairs.index((f_verts[i], f_verts[(i+1)%3]))])
                    elif (f_verts[(i+1)%3], f_verts[i]) in edge_pairs:
                        e.append(edges[edge_pairs.index((f_verts[(i+1)%3], f_verts[i]))])
                    else:
                        edge_pairs.append((f_verts[i], f_verts[(i+1)%3]))
                        edges.append(Edge(f_verts[i], f_verts[(i+1)%3]))
                        e.append(edges[-1])

                faces.append(Face(e[0],e[1],e[2]))

                if len(f_verts)>3:       
                    e=[]
                    print len(f_verts)
                    j = [0,2,3]
                    f_verts=[f_verts[j[i]] for i in range(3)]
                    for i in range(3):
                        if (f_verts[i], f_verts[(i+1)%3]) in edge_pairs:
                            e.append(edges[edge_pairs.index((f_verts[i], f_verts[(i+1)%3]))])
                        elif (f_verts[(i+1)%3], f_verts[i]) in edge_pairs:
                            e.append(edges[edge_pairs.index((f_verts[(i+1)%3], f_verts[i]))])
                        else:
                            edge_pairs.append((f_verts[i], f_verts[(i+1)%3]))
                            edges.append(Edge(f_verts[i], f_verts[(i+1)%3]))
                            e.append(edges[-1])

                    faces.append(Face(e[0],e[1],e[2]))

        #center on origin
        v_sum = Vector([0,0,0])
        for v in verts:
            v_sum += v.pos
        centroid = v_sum/len(verts)
        for v in verts:
            v.pos = v.pos-centroid    

        #gather naked edges for wire
        wire_verts=[]
        for e in edges:
            if len(e.faces)!=2:
                for i in range(2):
                    if not (e.vertices[i] in wire_verts):
                        wire_verts.append(e.vertices[i])
        wire = Wire(wire_verts)
        mesh = Mesh(verts, edges, faces)
        bub  = Bubble([wire], mesh) 
        mesh.set_bubble(bub)

        #split edges that span edge to edge
        for e in mesh.edges[:]:
            if (len(e.faces)==2) and (e.vertices[0] in wire.vertices) and (e.vertices[1] in wire.vertices) :
                mesh.split(e, Vertex(e.midpoint()))
        print 'done loading'

        return bub
                                
    def __init__(self, wires, mesh):
        self.wires = wires
        self.mesh = mesh





